#  Copyright (c) 2025 by the Eozilla team and contributors
#  Permissions are hereby granted under the terms of the Apache 2.0 License:
#  https://opensource.org/license/apache-2-0.

from .service_base import DEFAULT_CONFORMS_TO, ServiceBase

__all__ = [
    "DEFAULT_CONFORMS_TO",
    "ServiceBase",
]
