#  Copyright (c) 2025 by the Eozilla team and contributors
#  Permissions are hereby granted under the terms of the Apache 2.0 License:
#  https://opensource.org/license/apache-2-0.

from typing import Final

ENV_VAR_SERVER_HOST = "EOZILLA_SERVER_HOST"
ENV_VAR_SERVER_PORT = "EOZILLA_SERVER_PORT"
ENV_VAR_SERVICE: Final = "EOZILLA_SERVICE"

DEFAULT_HOST = "127.0.0.1"
DEFAULT_PORT = 8008
